/*
  8hz mp3 encoder BeOS GUI
*/

#include "Encoder.h"
#include "EncListItem.h"
#include "error.h"
#include "types.h"
#include "layer3.h"
#include "wave.h"
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <unistd.h>
#include <AppKit.h>
#include <KernelKit.h>
#include "Preference.h"
#include "MTextControl.h"

extern Preference Pref;

EncoderServer::EncoderServer(int threads, BListView *lv)
{
	if(threads==0)
		threads=1;
	maxthreads=threads;
	done=false;
	listview=lv;

	sem=create_sem(maxthreads,"encodersem");

}
EncoderServer::~EncoderServer()
{
	delete_sem(sem);
}

void EncoderServer::Start()
{
	resume_thread(spawn_thread(_encoderloop,"Encoder",B_LOW_PRIORITY,this));
}

void EncoderServer::Stop()
{
	done=true;
	acquire_sem_etc(sem,maxthreads,0,0);
}

void EncoderServer::SetThreadCount(long count)
{
	while(count>maxthreads)
	{
		release_sem(sem);
		maxthreads++;
	}
	while(count<maxthreads)
	{
	    // to avoid blocking the window, we acquire the semaphore from a seperate thread
		resume_thread(spawn_thread(_asyncacquiresem,"Async acquiresem",B_REAL_TIME_PRIORITY,this));
		maxthreads--;
	}
}

long EncoderServer::_asyncacquiresem(void *arg)
{
	((EncoderServer*)arg)->AsyncAcquireSem();
	return 0;
}
void EncoderServer::AsyncAcquireSem()
{
		acquire_sem(sem);
}

long EncoderServer::_encoderloop(void *arg)
{
	((EncoderServer*)arg)->EncoderLoop();
	return 0;
}

void acquire_sem_last(sem_id sem)
{
// loop until the semaphore can be acquired, this guarantees that
// normal calls to acquire_sem get acknowledged first.
	while(B_WOULD_BLOCK==acquire_sem_etc(sem,1,B_TIMEOUT,0))
		snooze(1E6);
}

void EncoderServer::EncoderLoop()
{
	while(!done)
	{
		char name[B_FILE_NAME_LENGTH+1];
		EncListItem *item;
		// find a new file to encode
		for(int i=0;;i++)
		{
			item=(EncListItem*)listview->ItemAt(i);
			if(!item)
				break;
			if(!item->isencoding)
			{
				item->isencoding=true;
				acquire_sem_last(sem);
				strcpy(name,item->filename);
				Encoder *enc=new Encoder(sem, name, listview, item);
				enc->Run();
			}
		}
		snooze(1E6);
	}
}

// =============================================

Encoder::Encoder(sem_id id, char *name, BListView *listview, EncListItem *listitem)
{
	sem=id;
	filename=strdup(name);
	lv=listview;
	item=listitem;
}

void Encoder::Run()
{
	resume_thread(spawn_thread(_encoderfunc,"Encoder",B_LOW_PRIORITY,this));
}

long Encoder::_encoderfunc(void *arg)
{
	((Encoder*)arg)->EncoderFunc();
	return 0;
}

void Encoder::EncoderFunc()
{
	// load image
	app_info info; 
	if (be_app->GetAppInfo(&info)!=B_OK) 
		ERROR("Couldn't find myself");
	BEntry entry(&info.ref);
	BPath path;
	entry.GetPath(&path);
	image_id image=load_add_on(path.Path());
	if(image<0)
		ERROR("Error loading self");

	// find symbols
	void (*funcptr)(char *name,char *destination,long bitrate, bool del, bool copyright, bool original, BListView *lv, EncListItem *item);
	if(B_OK!=get_image_symbol(image, "DoCompression", B_SYMBOL_TYPE_TEXT, (void**)&funcptr))
		ERROR("Couldn't find symbol 'DoCompression', check exports");

	printf("getting destination path\n");
	extern MTextControl *destpath;
	char *destination=(char*)destpath->Text();
	printf("destination path: %s\n",destination);
	

	// encode
	funcptr(filename,destination,Pref.FindInt32("bitrate"),
					Pref.FindBool("delete"),
					Pref.FindBool("copyright"),
					Pref.FindBool("original"),
					lv,item
					);

	// unload image
	unload_add_on(image);

	// clean up rest
	release_sem(sem);
	free(filename);
	delete this;
}

extern "C" void set_defaults();
extern "C" void check_config();
extern "C" void updatefunc(int);

EncListItem *item;
BListView *lv;

void DoCompression(char *name, char *destination,long bitrate, bool del, bool copyright, bool original, BListView *listview, EncListItem *listitem)
{
	item=listitem;
	lv=listview;
	set_defaults();
	config.infile=name;
	char mp3name[B_FILE_NAME_LENGTH+B_PATH_NAME_LENGTH+5];

	if((destination==NULL)||(strlen(destination)==0))
		strcpy(mp3name,name);
	else
	{
		strcpy(mp3name,destination);
		char *s=name+strlen(name)-1;
		while((s>name)&&(*s!='/'))
			s--;
		strcat(mp3name,s);
	}
	printf("mp3name: %s\n",mp3name);
	char *ext=mp3name+strlen(mp3name);
	if(strlen(mp3name)>4)
	{
		if(strcasecmp(ext-4,".wav")==0)
			ext-=4;
	}
	if(strlen(mp3name)>5)
	{
		if(strcasecmp(ext,".aiff")==0)
			ext-=5;
	}
	if(strlen(mp3name)>6)
	{
		if(strcasecmp(ext-6,".sound")==0)
			ext-=6;
	}
	strcpy(ext,".mp3");

	config.outfile=mp3name;
	if(!wave_open()) ERROR("Unable to open input file...");

	config.mpeg.bitr=bitrate;
	config.mpeg.copyright=copyright;
	config.mpeg.original=original;
	
	check_config();
	L3_compress(updatefunc);
	updatefunc(config.mpeg.total_frames);
	wave_close();
	if(del)
		unlink(name);

}

void updatefunc(int framesdone)
{
	char buf[B_FILE_NAME_LENGTH+B_PATH_NAME_LENGTH+10];
	sprintf(buf,"(%2.2f%% done) %s", 
    	        (float)((float)framesdone/config.mpeg.total_frames)*100,
				config.infile
				); 
	lv->Window()->Lock();
	item->SetText(buf);
	lv->InvalidateItem(lv->IndexOf(item));
	lv->Window()->Unlock();
}
