/*
  8hz mp3 encoder BeOS GUI
*/


#include "GUI.h"
#include "Encoder.h"
#include "EncListItem.h"
#include "Preference.h"
#include "BrowseButton.h"

#include "layout-all.h"
#include "StorageKit.h"

// mpeg config settings, PrefServer style. Easy!
Preference Pref;

EncoderServer *encserv;
MTextControl *destpath;

class MP3ListView: public MListView
{

	public:
		MP3ListView()
			: MListView()
		{}

		void AddDirToList(const entry_ref *ref, BList *list)
		{
			BDirectory dir(ref);
			entry_ref childref= *ref;

			while(dir.GetNextRef(&childref)!=B_ENTRY_NOT_FOUND)
			{
				BEntry entry(&childref);
				if(entry.IsFile())
					list->AddItem(new EncListItem(&childref));
				else if(entry.IsDirectory())
					AddDirToList(&childref,list);
				else if(entry.IsSymLink())
				{
					BEntry linkentry(&childref,true);
					if(linkentry.IsFile())
						list->AddItem(new EncListItem(&childref));
					else if(linkentry.IsDirectory())
					{
						entry_ref linkref;
						linkentry.GetRef(&linkref);
						AddDirToList(&linkref,list);
					}
				}
			}
		}



	virtual void MessageReceived(BMessage *mes)
	{
		BList list;
		long insertpoint;
		ulong type;
		long count;

		switch(mes->what)
		{
			case B_SIMPLE_DATA:
				insertpoint=CountItems();
				mes->GetInfo("refs", &type, &count);
				for(long i=0; i<count ;i++)
				{
					entry_ref item;
					mes->FindRef("refs",i,&item);
					BEntry entry=BEntry(&item,true);
					BNode node;
					char attribute[11];

					if(entry.IsFile())
					{
						list.AddItem(new EncListItem(&item));
					}
					else if(entry.IsDirectory())
						AddDirToList(&item, &list);
				}
				AddList(&list,insertpoint);
				break;



			default:
				MListView::MessageReceived(mes);
				break;
		}
	}

	virtual void AttachedToWindow()
	{
		encserv=new EncoderServer(Pref.FindInt32("concurrency"),this);
		encserv->Start();
	}

};

static long mpeg1[15] = {32,40,48,56,64,80,96,112,128,160,192,224,256,320};

class EncoderWin: public MWindow
{
	private:
		MButton *startbutton;
		bool	isrunning;

	public:

		EncoderWin()
			:MWindow(BRect(100,100,550,350),"mp3 encoder",B_TITLED_WINDOW,0)
		{
			isrunning=false;
			MPopup *concpop;
			MPopup *ratepop;

			// Add sensible defaults to preference
			Pref.AddString("destpath","");
			Pref.AddBool("delete",false);
			Pref.AddInt32("bitrate",128);
			Pref.AddInt32("concurrency",2);
			Pref.AddBool("original",false);
			Pref.AddBool("copyright",false);
			Pref.Load();
			// Tadaaaa! Preferences are now loaded. How's that for only a few lines of code?

			destpath=new MTextControl("Destination folder:",(char *)Pref.FindString("destpath"));
			AddChild
			(
				new MBorder
				(M_RAISED_BORDER,3,"",
					new VGroup
					(
						new VGroup
						(
							new MBorder
							(
								M_LABELED_BORDER,8,"Files",
								new MScrollView(new MP3ListView(),false,true)
							),
							new MBorder
							(
								M_LABELED_BORDER,8,"Settings",
								new VGroup
								(
									new HGroup
									(
										concpop=new MPopup("Concurrent encodes",new BMessage('conc'),this,"1","2","3","4",0),
										new MCheckBox("Set Original bit",new BMessage('orig'),this,Pref.FindBool("original")),
										0
									),
									new HGroup
									(
										ratepop=new MPopup("bitrate (kilobit/sec)",new BMessage('rate'),this,"32","40","48","56","64","80","96","112","128","160","192","224","256","320",0),
										new MCheckBox("Set Copyright bit",new BMessage('copy'),this,Pref.FindBool("copyright")),
										0
									),

									new HGroup
									(
										destpath,
										new BrowseButton(destpath),
										0
									),

									new MCheckBox("Delete sourcefiles after encoding",new BMessage('dele'),this,Pref.FindBool("delete")),
									0
								)
							),
							0
						),
						new MStringView("8hz mp3 encoder 0.2b. BeOS version 1.1"),
/*
						new HGroup
						(
							new MButton("Stop",'stop',minimax(-1,-1,-1,-1)),
							startbutton=new MButton("Start",'strt',minimax(-1,-1,-1,-1)),
							0
						),
*/
						0
					)
				)
			);

			// make UI reflect current settings
			long rate=Pref.FindInt32("bitrate");
			int i;
			for(i=0;i<13;i++)
			{
				if(mpeg1[i]==rate)
					break;
			}
			ratepop->SetActive(i,false);
			long conc=Pref.FindInt32("concurrency");
			if((conc<=0) || (conc>=5))
				conc=2;
			concpop->SetActive(conc-1,false);

			DivideSame(ratepop,concpop,0);

		}

	virtual ~EncoderWin()
	{
		// save settings on exit.
		Pref.ReplaceString("destpath",destpath->Text());
		Pref.Save();
	}

	virtual bool QuitRequested()
	{
		be_app->PostMessage(B_QUIT_REQUESTED);
		return true;
	}

	virtual void MessageReceived(BMessage *mes)
	{
		switch(mes->what)
		{
			case M_BUTTON_SELECTED:
				if(mes->FindInt32(M_BUTTON_ID)=='strt')
				{
					if(isrunning)
					{
						isrunning=false;
						startbutton->SetLabel("Start");
					}
					else
					{
						isrunning=true;
						startbutton->SetLabel("Pause");
					}
				}
			case 'copy':
				Pref.ReplaceBool("copyright",mes->FindInt32("be:value"));
				break;

			case 'orig':
				Pref.ReplaceBool("original",mes->FindInt32("be:value"));
				break;

			case 'dele':
				Pref.ReplaceBool("delete",mes->FindInt32("be:value"));
				break;

			case 'rate':
				Pref.ReplaceInt32("bitrate",mpeg1[mes->FindInt32("index")]);
				break;

			case 'conc':
				{
					long newthreads=mes->FindInt32("index")+1;
					Pref.ReplaceInt32("concurrency",newthreads);
					encserv->SetThreadCount(newthreads);
				}
				break;

			default:
				MWindow::MessageReceived(mes);
				break;
		}
	}
};

DoGUI()
{
	MApplication app("application/x-vnd.8hz-mp3enc");

	EncoderWin *win=new EncoderWin;

	win->Show();

	app.Run();
	return 0;
}
