/*
  8hz mp3 file reader
*/

#include <stdio.h>
#include "portableio.h"

#include "types.h"
#include "error.h"
#include "wave.h"
#include "string.h"
#include "stdlib.h"

#include "SoundFile.h"

BSoundFile *soundfile;

void wave_close()
{
    delete soundfile;
}

bool wave_open()
{
    static char    *channel_mappings[] = {NULL,"mono","stereo"};

	entry_ref soundref;
	get_ref_for_path(config.infile,&soundref);

	soundfile=new BSoundFile(&soundref,B_READ_ONLY);
	if(soundfile->InitCheck())
    	ERROR("Unable to open file");

    config.wave.type = soundfile->FileFormat();

    config.wave.channels   = soundfile->CountChannels();
    config.wave.samplerate = soundfile->SamplingRate();
	config.wave.byteorder  = soundfile->ByteOrder();

    if(config.wave.channels>2)
		ERROR("More than 2 channels\n");
    printf("%s %ldHz ", channel_mappings[config.wave.channels], config.wave.samplerate);
    config.wave.bits       = soundfile->SampleSize()*8;
    if(config.wave.bits!=16)
		ERROR("NOT 16 Bit !!!\n");
    printf("%dbit, ", config.wave.bits);

    config.wave.total_samples = soundfile->CountFrames();
    config.wave.length = config.wave.total_samples/config.wave.samplerate;
    printf("Length: %2ld:%2ld:%2ld\n", (config.wave.length/3600), 
                                       (config.wave.length/60)%60,
                                       (config.wave.length%60)); 
    return true;
}

static int read_samples(short *sample_buffer,
                        int    num16bitsamples)
{
    int samples_read;
	int factor=soundfile->FrameSize()/2; // 1 for mono, 2 for stereo

	samples_read=soundfile->ReadFrames((char*)sample_buffer,num16bitsamples/factor)*(factor);
	if((config.wave.byteorder==B_LITTLE_ENDIAN && config.byte_order==order_bigEndian) ||
	   (config.wave.byteorder==B_BIG_ENDIAN && config.byte_order==order_littleEndian))
	{
               SwapBytesInWords(sample_buffer,samples_read);
	}
    if(samples_read<num16bitsamples && samples_read>0) 
    { /* Pad sample with zero's */
        while(samples_read<num16bitsamples) sample_buffer[samples_read++] = 0;
    }
    return samples_read;
}


int wave_get(short buffer[2][1152])
/* expects an interleaved 16bit pcm stream from read_samples, which it */
/* de-interleaves into buffer                                          */
{
    static short temp_buf[2304];
    int          samples_read;
    int          j;

    switch(config.mpeg.mode)
    {
        case MODE_MONO  :
            samples_read = read_samples(temp_buf,config.mpeg.samples_per_frame);
            for(j=0;j<1152;j++)
            {
                buffer[0][j] = temp_buf[j];
                buffer[1][j] = 0;
            }
            break;

        case MODE_STEREO : 
        case MODE_JSTEREO:
        case MODE_DUAL   :
            samples_read = read_samples(temp_buf,config.mpeg.samples_per_frame*2);
            for(j=0;j<1152;j++)
            {
                buffer[0][j] = temp_buf[2*j];
                buffer[1][j] = temp_buf[2*j+1];
            }
            break;

        default          :
            ERROR("[wav_get], channel mode not supported");
    }
    return samples_read;
}
 
